package code;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.util.ArrayList;

import java.util.List;
import org.apache.commons.codec.binary.Base64;

import com.sun.net.httpserver.HttpExchange;
import com.sun.net.httpserver.HttpHandler;

public class RedirectPage implements HttpHandler{
	
	private KeyCloak kcs;
	
	public RedirectPage(KeyCloak kcs) {
		this.kcs = kcs;
	}

	@Override
    public void handle(HttpExchange exchange) throws IOException {
        URI requestURI = exchange.getRequestURI();
        boolean wantsRedirectPage = Helper.compareText(requestURI.toString(),URI.create("/").toString());
        if(!wantsRedirectPage) {
            String error = "Invalid URI";
            OutputStream os = exchange.getResponseBody();
            exchange.sendResponseHeaders(400, error.getBytes().length);
            os.write(error.getBytes());
            os.close();
            return;
        }

        String requestMethod = exchange.getRequestMethod();
        if (Helper.compareText(requestMethod, "GET")) {
        	
        	String codeVerifier = createCodeVerifier();
        	try {
				String codeChallenge = createCodeChallenge(codeVerifier);
				String state = "baf78db5-4c5b-4d56-8111-8fe63c38961a"; //An opaque arbitrary alphanumeric string your app adds to the initial request that Auth0 includes when redirecting back to your application. 
				String nonce = "a81e1a84-8885-4702-b8d1-f6c5a0d1fc4d";
				System.out.println("CODE VERIFIER = "+codeVerifier);
				// get the html page
	            List<String> strlist = new ArrayList<>();
	            String response = null;
	            response = getRedirectPage();
	            strlist.add("text/html");
	            
	            if(response != null && !Helper.compareText(response, "fail")){
	            	response = response.replace("$DOMAIN", kcs.authServer())
	            			.replace("$REALM", kcs.realm())
	            			.replace("$MY_CODE_CHALLENGE", codeChallenge)
	            			.replace("$MY_CLIENT_ID", kcs.clientId())
	            			.replace("$MY_REDIRECT_URI", kcs.redirectUri())
	            			.replace("$MY_NONCE",nonce)
	            			.replace("$MY_STATE", state);	            	
	            	System.out.println(response);
	                exchange.getResponseHeaders().put("content-type", strlist);
	                exchange.sendResponseHeaders(200, response.getBytes().length);
	                OutputStream os = exchange.getResponseBody();
	                os.write(response.getBytes());
	                os.close();
	            } else {
	                exchange.sendResponseHeaders(500, response.getBytes().length);
	                OutputStream os = exchange.getResponseBody();
	                os.write(response.getBytes());
	                os.close();
	            }
				
			} catch (UnsupportedEncodingException | NoSuchAlgorithmException e) {
				System.out.println("Error during creation of code challenge");
			}        	
        } else {
            Helper.methodNotAllowed(exchange);
        }
    }

    

	private static String getRedirectPage() {
        String line;
        String page = Server.CLIENT_PATH+"/redirect.html";

        StringBuilder answer = new StringBuilder();
        if (getExtension(page).length() == 0)
            page += ".html";

        BufferedReader bufferedReader = null;
        try {
            FileReader fileReader = new FileReader(page);

            bufferedReader = new BufferedReader(fileReader);
            boolean isComment = false;
            while ((line = bufferedReader.readLine()) != null) {
            	line = line.trim();

            	if(line.startsWith("<!--") && line.endsWith("-->")) {
            		continue;
            	}
            	if(line.startsWith("<!--")) {
            		isComment = true;
            		continue;
            	}
            	if(line.endsWith("-->")) {
            		isComment = false;
            		continue;
            	}

            	if(!isComment && line.length()>0)
                	answer.append(line).append("\n");
            }
        } catch (FileNotFoundException ex) {
            System.out.println("Unable to open file '" + page + "'");
            return "fail";
        } catch (IOException ex) {
            System.out.println("Error reading file '" + page + "'");
            return "fail";
        } finally {
            try{
                if(bufferedReader != null)
                    bufferedReader.close();
            } catch (IOException ex){
                System.out.println("Error closing bufferedReader");
            }
        }
        return answer.toString();
    }


    private static String getExtension(String file) {
        int i = file.length() - 1;
        while (i > 0 && file.charAt(i) != '.' && file.charAt(i) != '/')
            i--;
        if (file.charAt(i) == '.')
            return file.substring(i + 1);
        else
            return "";
    }
    
    
    private String createCodeVerifier() {
    	SecureRandom sr = new SecureRandom();
    	byte[] code = new byte[32];
    	sr.nextBytes(code); 
    	return java.util.Base64.getUrlEncoder().withoutPadding().encodeToString(code);
    }
    
    
    private String createCodeChallenge(String verifier) throws UnsupportedEncodingException, NoSuchAlgorithmException {
    	byte[] bytes = verifier.getBytes("US-ASCII");
    	MessageDigest md = MessageDigest.getInstance("SHA-256");
    	md.update(bytes, 0, bytes.length);
    	byte[] digest = md.digest();
    	return Base64.encodeBase64URLSafeString(digest);
	}

}